"""
Name: EK_AF_UCS_Classifier.py
Authors: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA
Contact: ryan.j.urbanowicz@darmouth.edu
Created: Feb. 12, 2010
---------------------------------------------------------------------------------------------------------------------------------------------------------
EK_AF_UCS Version 1.0: An learning classifier system algorithm implemented with our new attribute tracking, feedback, and expert knowledge strategies for
modeling epistasis and heterogeneity data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

#!/usr/bin/env python
#Import modules
from EK_AF_UCS_Constants import *
from EK_AF_UCS_SNP_Environment import *
from random import *
import math
import copy

class Classifier:  
    #Set Constructors------------------------------------------------------------------------------------------------------- 
    def __init__(self,a=None,b=None,c=None,d=None):
        """ Overloaded constructor: constructs a classifier based on the type of arguments. """
        self.condition = []
        self.classification = None
        self.correctTrack = 0
        self.accuracy = 0.0
        self.fitness = 0.0
        self.numerosity = 1
        self.experience = 0.0
        self.actionSetSize = 1
        self.timeStamp = 0
        self.initTimeStamp = 0
        self.cl_generality = 0.0
        
        #Final Eval. Track
        self.evalTrainMatch = 0
        self.evalTrainCorrect = 0
        self.evalTestMatch = 0
        self.evalTestCorrect = 0
        
        #Coverage Fitness
        self.coverage = None
          
        if isinstance(c,list):
            self.makeMatchingClassifier(a,b,c,d)
        elif isinstance(a,Classifier):
            self.makeCopyClassifier(a, b)
        elif isinstance(a,list) and b == None:
            self.makeRemakeClassifier(a)
        else:
            print "Classifier: Correct arguments not found."
            
            
    def makeMatchingClassifier(self, setSize, exploreIter, cond, clss):
        """ Constructs a classifier with matching condition and specified action. """
        self.createMatchingCondition(cond)
        self.classification = clss
        self.classifierSetVariables(setSize, exploreIter)
        
        
    def makeCopyClassifier(self,clOld, exploreIter):
        """  Constructs an identical Classifier.  However, the experience of the copy is set to 0 and the numerosity 
        is set to 1 since this is indeed a new individual in a population. """
        self.condition = copy.deepcopy(clOld.condition) # a copy of the condition not the same reference
        self.classification = clOld.classification
        self.correctTrack = 0   #does this belong here?? Double check.
        self.accuracy = cons.accuracyIni
        self.fitness = cons.fitnessIni
        self.numerosity    = 1
        self.experience    = 0
        self.actionSetSize = clOld.actionSetSize
        self.timeStamp     = exploreIter
        self.initTimeStamp = exploreIter
    
    
    def makeRemakeClassifier(self,classifierList): #START HERE - get file position for initTimeStamp, and put it in the output file  #also fix resetting of experience and correct (make it last)
        """ Re-Assigns all classifier values. """
        attRef = cons.environmentObject.numAttributes
        #Remake Condition
        for i in range(0,attRef):
            self.condition.append(str(classifierList[i]))
        if len(self.condition) != attRef:
            print " Error in remaking Classifier!"
            
        self.classification = str(classifierList[attRef])
        self.fitness = float(classifierList[attRef+1])
        self.accuracy = float(classifierList[attRef+2])
        self.correctTrack = int(classifierList[attRef+3])
        self.experience = float(classifierList[attRef+4])
        self.numerosity = int(classifierList[attRef+5])
        self.actionSetSize = float(classifierList[attRef+6])
        self.timeStamp = int(classifierList[attRef+7])
        self.cl_generality = float(classifierList[attRef+8])
        self.initTimeStamp = int(classifierList[attRef+15])
    #----------------------------------------------------------------------------------------------------------------- 
    
    
    #For selecting a single attribute for general covering (main effect search)*************************************** 
    def selectAttribute(self, probSum, probList):
        """ Selects one classifier using roulette wheel selection according to the fitness of the classifiers. """
        choiceP = random()*probSum
        i=0
        sum = probList[i]
        while choiceP>sum:
            i=i+1
            sum = sum + probList[i]
        return i

        
    def createMatchingCondition(self, cond):
        """ Creates a matching condition. """  
        condArray  = []
        #print "creating matching condition"
        if cons.expertK != 'False' and cons.EK_type == 'simpleTransformation' and (cons.EK_apply == 'cover' or cons.EK_apply == 'both'):  #Uses EK
            #pick number of attributes to specify
            
            refList = []
            for i in range(len(cond)):
                refList.append(i)
                
            attID = []

            specAtts = randint(1,int(len(cond)*cons.coverGenerality))
            #pick the attributes
            
            EK = copy.deepcopy(cons.EK.weightList)
            for i in range(specAtts):
                returnedAtt = self.selectAttribute(sum(EK),EK)
                attID.append(refList[returnedAtt])
                EK.pop(returnedAtt)
                refList.pop(returnedAtt)
            #print attID
            for i in range(len(cond)):
                if i in attID:
                    condArray.append(cond[i])
                else:
                    condArray.append(cons.wild)
            #print condArray
            self.condition = condArray  
            
        elif cons.expertK != 'False' and cons.EK_type == 'simpleLogit' and (cons.EK_apply == 'cover' or cons.EK_apply == 'both'):
            condArray  = []
            for i in range(len(cond)):
                if random() < cons.EK.weightList[i]:
                    condArray.append(cond[i])
                else:
                    condArray.append(cons.wild)
            self.condition = condArray
            
        elif cons.expertK != 'False' and cons.EK_type == 'newLogit' and (cons.EK_apply == 'cover' or cons.EK_apply == 'both'):
            condArray  = []
            for i in range(len(cond)):
                if random() < cons.EK.weightList[i]:
                    condArray.append(cond[i])
                else:
                    condArray.append(cons.wild)
            self.condition = condArray
            
        else:  #no expert knowledge
            #pick number of attributes to specify
            specAtts = randint(1,int(len(cond)*cons.coverGenerality))
            #print specAtts
            singleProb = specAtts/float(len(cond))
            #pick the attributes
            attSpecified = False
            for i in range(len(cond)):
                if random() < singleProb:
                    condArray.append(cond[i])
                    attSpecified = True
                else:
                    condArray.append(cons.wild)
            if not attSpecified:
                tempKey = randint(0,len(cond)-1)
                condArray[tempKey] = cond[tempKey]
            #print condArray
            self.condition = condArray               
                                  

    def classifierSetVariables(self, setSize, exploreIter):
        """ Sets the initial variables of a new classifier. """
        self.accuracy = cons.accuracyIni
        self.fitness = cons.fitnessIni
        self.correctTrack = 0
        self.numerosity = 1
        self.experience = 0
        self.actionSetSize = setSize
        self.timeStamp = exploreIter
        self.initTimeStamp = exploreIter
        
        
    def match(self, state):
        """ Returns if the classifier matches in the current situation. """ #Dictionary Update
        if len(self.condition) != len(state):
            print "Error in length of either rule condition or state length"
            return False
        for i in range(len(self.condition)):
            if self.condition[i] != cons.wild and self.condition[i]!=state[i]:
                if state[i] == '#': #missing data catch
                    pass
                else:
                    return False;
        return True

    
    def isAllWild(self):
        """  Checks to see if the condition is completely wild """ #Dictionary Updated
        wildCount = 0
        for i in range(len(self.condition)):
            if self.condition[i] == cons.wild:
                wildCount += 1
        if wildCount == len(self.condition):
            return True
        else:
            return False
            
            
    #GA Operators--------------------------------------------------------------------------
    def twoPointCrossover(self,cl):
        """ Applies two point crossover and returns if the classifiers changed. """
        changed = False;
        length = len(self.condition)
        sep1   = int(random()*(length))
        sep2   = int(random()*(length)) + 1
        if sep1>sep2:
            help1 = sep1
            sep1 = sep2
            sep2 = help1

        elif sep1==sep2:
            sep2= sep2+1

        cond1 = self.condition[:]
        cond2 = cl.condition
        for i in range(sep1,sep2):
            if cond1[i] != cond2[i]:
                changed = True
                help2   = cond1[i]
                cond1[i]= cond2[i]
                cond2[i]= help2

        if changed:
            self.condition = cond1
            cl.condition   = cond2

        return changed


    def singlePointCrossover(self, cl):
        """ Applies single point crossover and returns if the classifiers changed. """
        changed = False;
        length = len(self.condition)
        sep = int(random()*(length))

        cond1 = self.condition[:]
        cond2 = cl.condition
        for i in range(sep):
            if cond1[i] != cond2[i]:
                changed = True
                temp = cond1[i]
                cond1[i]= cond2[i]
                cond2[i]= temp

        if changed:
            self.condition = cond1
            cl.condition   = cond2

        return changed
        
        
    def uniformCrossover(self, cl, env, percent):
        """ Applies uniform crossover and returns if the classifiers changed.  """ #Dictionary rule 
        if cons.attributeFeedback and cons.attributeTracking and random() < percent:  #Attribute Tracking used to guide crossover
            trackList = self.makeTrackWeight(env) #make the weight list 

            minVal = min(trackList)
            for i in range(len(trackList)):
                trackList[i] = trackList[i] - minVal
            maxVal = max(trackList)

            changed = False;   
            cond1 = self.condition[:]
            cond2 = cl.condition

            #cond1 is the target for the better attributes
            for i in range(len(self.condition)):
                if maxVal == 0.0:
                    probability = 0.5
                else:
                    probability = trackList[i]/float(maxVal + maxVal*cons.EK_prob_adjust)

                if cond1[i] != cond2[i]:
                    if cond1[i] != cons.wild and random() > probability:  #if specified att and high prob mean less likely to swap
                        changed = True
                        temp = cond1[i]
                        cond1[i]= cond2[i]
                        cond2[i]= temp
                    if cond1[i] == cons.wild and random() < probability: 
                        changed = True
                        temp = cond1[i]
                        cond1[i]= cond2[i]
                        cond2[i]= temp
    
            if changed:
                self.condition = cond1
                cl.condition   = cond2
                
        else:  #Original Mechanism
            changed = False;   
            cond1 = self.condition[:]
            cond2 = cl.condition
            for i in range(len(self.condition)):
                if cond1[i] != cond2[i] and random() < 0.5:
                    changed = True
                    temp = cond1[i]
                    cond1[i]= cond2[i]
                    cond2[i]= temp
    
            if changed:
                self.condition = cond1
                cl.condition   = cond2

        return changed
         
             
    def halfUniformCrossover(self, cl):
        """ Applies half uniform crossover and returns if the classifiers changed. """
        changed = False;   
        cond1 = self.condition[:]
        cond2 = cl.condition
        diffList = []
        for i in range(len(self.condition)):
            if cond1[i] != cond2[i]:
                diffList.append(i)
                
        swapCount = int(len(diffList) / 2.0)
        
        for i in range(len(swapCount)):
            target = diffList.pop(random.randint(0,len(diffList) -1))                
            changed = True
            temp = cond1[target]
            cond1[target]= cond2[target]
            cond2[target]= temp

        if changed:
            self.condition = cond1
            cl.condition   = cond2

        return changed
        
        
    def makeLocalCondition(self):
        """ Attempts to find a new local condition with the same specified attributes - i.e. no change in wilds - Part of Local Predictive Rule Search """ #Dictionary Update
        attributeList = ['0','1','2'] #Put here temporarily: in final implementation this should be detected in the environment.
        for i in range(len(self.condition)):
            if self.condition[i] != cons.wild:
                self.condition[i] = attributeList[randint(0,len(attributeList)-1)]
    
    
    def switchClass(self):
        """ Changes the classifiers classification.- Part of Local Predictive Rule Search """
        if self.classification  == '1':
            self.classification = '0'
        else:
            self.classification = '1'
        
        
    def applyMutation(self, state, env, percent):
        """ Applies a niche mutation to the classifier.
        This method calls mutateCondition(state) and mutateAction(numberOfActions) and returns
        if at least one bit or the action was mutated. """
        changed = self.mutateCondition(state, env, percent)
        if self.mutateAction(cons.numClasses):
            changed = True
        return changed

        
    def makeTrackWeight(self, env):
        trackList = []
        if cons.attributePunish:
            if cons.attDiv:
                tempList = copy.deepcopy(env.getAttTrack())
                tempListCount = copy.deepcopy(env.getCountTrack())
                tempListPun = copy.deepcopy(env.getAttPunTrack())
                tempListCountPun = copy.deepcopy(env.getCountPunTrack())
                corList = []
                punList = []
                for i in range(len(tempList)):
                    if tempListCount[i] == 0:
                        corList.append(0)
                    else:
                        corList.append(tempList[i]/float(tempListCount[i]))
                for i in range(len(tempListPun)):
                    if tempListCountPun[i] == 0:
                        punList.append(0)
                    else:
                        punList.append(tempListPun[i]/float(tempListCountPun[i]))
                for i in range(len(corList)):
                    trackList.append(corList[i]-punList[i])
            else:
                tList = copy.deepcopy(env.getAttTrack()) 
                tList2 = copy.deepcopy(env.getAttPunTrack())
                for i in range(len(tList)):
                    trackList.append(tList[i]-tList2[i])
        else:
            if cons.attDiv:
                tempList = copy.deepcopy(env.getAttTrack())
                tempListCount = copy.deepcopy(env.getCountTrack())
                for i in range(len(tempList)):
                    if tempListCount[i] == 0:
                        trackList.append(0)
                    else:
                        trackList.append(tempList[i]/float(tempListCount[i]))
            else:
                trackList = copy.deepcopy(env.getAttTrack())
        return trackList
        
        
    def mutateCondition(self, state, env, percent): 
        """ Mutates the condition of the classifier. If one allele is mutated depends on the constant pM.
        This mutation is a niche mutation. It assures that the resulting classifier still matches the current situation. """ # Dictionary Update
        changed    = False
        condLength = len(self.condition)
        cond = self.condition[:]

        if cons.expertK != 'False' and cons.EK_type == 'simpleTransformation' and (cons.EK_apply == 'mutation' or cons.EK_apply == 'both'): #There is expert knowledge
            if cons.attributeTracking and cons.attributeFeedback and random() < percent: #Use Attribute Tracking for mutation
                trackList = self.makeTrackWeight(env) #make the weight list 

                minVal = min(trackList)
                for i in range(len(trackList)):
                    trackList[i] = trackList[i] - minVal
                maxVal = max(trackList)
                for i in range(condLength):
                    if random() < cons.pMut:
                        if maxVal == 0.0:
                            probability = 0.5
                        else:
                            probability = trackList[i]/float(maxVal + maxVal*cons.EK_prob_adjust)
                        if cond[i] == cons.wild and random() < probability: #if the EK is high there's a higher chance of specifying the att.
                            cond[i] = state[i]
                            changed = True
                        elif cond[i] != cons.wild and random() > probability: #if the EK is low there's a higher chance of generalizing the att
                            cond[i] = cons.wild
                            changed = True
                        else:
                            pass   
                        
            else: #Use EK weights for mutation
                for i in range(condLength):
                    if random() < cons.pMut:
                        probability = cons.EK.weightList[i]/float(cons.EKListMax +cons.EKListMax*cons.EK_prob_adjust)
                        if cond[i] == cons.wild and random() < probability: #if the EK is high there's a higher chance of specifying the att.
                            cond[i] = state[i]
                            changed = True
                        elif cond[i] != cons.wild and random() > probability: #if the EK is low there's a higher chance of generalizing the att
                            cond[i] = cons.wild
                            changed = True
                        else:
                            pass    
                        

        elif cons.expertK == 'False' and cons.attributeTracking and cons.attributeFeedback and random() < percent: #There is expert knowledge
            trackList = self.makeTrackWeight(env) #make the weight list        

            minVal = min(trackList)
            for i in range(len(trackList)):
                trackList[i] = trackList[i] - minVal
            maxVal = max(trackList)
            for i in range(condLength):
                if random() < cons.pMut:
                    if maxVal == 0.0:
                        probability = 0.5
                    else:
                        probability = trackList[i]/float(maxVal + maxVal*cons.EK_prob_adjust)
                    if cond[i] == cons.wild and random() < probability: #if the EK is high there's a higher chance of specifying the att.
                        cond[i] = state[i]
                        changed = True
                    elif cond[i] != cons.wild and random() > probability: #if the EK is low there's a higher chance of generalizing the att
                        cond[i] = cons.wild
                        changed = True
                    else:
                        pass   



        elif cons.expertK != 'False' and cons.EK_type == 'simpleLogit' and (cons.EK_apply == 'mutation' or cons.EK_apply == 'both'): #There is expert knowledge
            condLength = len(self.condition)
            cond = self.condition[:]
            for i in range(condLength):
                if random() < cons.pMut:
                    if cond[i] == cons.wild and random() < cons.EK.weightList[i]: 
                        cond[i] = state[i]
                        changed = True
                    elif cond[i] != cons.wild and random() > cons.EK.weightList[i]:
                        cond[i] = cons.wild
                        changed = True
                    else:
                        pass 

        elif cons.expertK != 'False' and cons.EK_type == 'newLogit' and (cons.EK_apply == 'mutation' or cons.EK_apply == 'both'): #There is expert knowledge
            condLength = len(self.condition)
            cond = self.condition[:]
            for i in range(condLength):
                if random() < cons.pMut:
                    if cond[i] == cons.wild and random() < cons.EK.weightList[i]: 
                        cond[i] = state[i]
                        changed = True
                    elif cond[i] != cons.wild and random() > cons.EK.weightList[i]:
                        cond[i] = cons.wild
                        changed = True
                    else:
                        pass 

        else: #no expert knowledge
            for i in range(condLength):
                if random() < cons.pMut:
                    changed = True
                    if cond[i] == cons.wild:
                        cond[i] = state[i]
                    else:
                        cond[i] = cons.wild

        self.condition = cond
        return changed
        

    def mutateAction(self, numClasses):
        """ Mutates the action of the classifier. - Makes it so a rule's class is less likely to mutate if the rule has high accuracy. """
        changed = False
        if random() < cons.pMut and random() > self.accuracy:
            act = int(random() * numClasses)
            while str(act) == str(self.classification):
                act = int(random() * numClasses)

            self.classification = str(act)
            changed= True
        return changed   
    #-------------------------------------------------------------------------------------
    
    #Subsumption Methods------------------------------------------------------------------
    def equals(self, cl):
        """ Returns if the two classifiers are identical in condition and action. """ # Dictionary Update
        if cl.condition==self.condition:
            if str(cl.classification) == str(self.classification):
                return True
        return False
    
    
    def subsumes(self, cl):
        """ Returns if the classifier subsumes cl. """
        if cl.classification == self.classification:
            if self.isSubsumer() and self.isMoreGeneral(cl):
                return True
        return False

        
    def isSubsumer(self):
        """ Returns if the classifier is a possible subsumer. It is affirmed if the classifier
        has a sufficient experience and if its reward prediction error is sufficiently low.  """
        if self.experience > cons.theta_sub  and self.accuracy > cons.accZ:
            return True
        return False

        
    def isMoreGeneral(self,cl):
        """ Returns if the classifier is more general than cl. It is made sure that the classifier is indeed more general and
        not equally general as well as that the more specific classifier is completely included in the more general one (do not specify overlapping regions) """ #Dictionary Update
        ret    = False
        length = len(self.condition)
        for i in range(length):
            if self.condition[i] != cons.wild and self.condition[i] != cl.condition[i]:
                return False
            elif self.condition[i] !=  cl.condition[i]:
                ret = True
        return ret
        
    #-------------------------------------------------------------------------------------
    
    def getDelProp(self, meanFitness):
        """  Returns the vote for deletion of the classifier. """
        if self.fitness/self.numerosity >= cons.delta*meanFitness or self.experience < cons.theta_del:
            return self.actionSetSize*self.numerosity
        # fitness was becoming zero
        if self.fitness == 0.0:
            return self.actionSetSize * self.numerosity * meanFitness / (cons.fitnessIni/self.numerosity)
        else:
            return self.actionSetSize * self.numerosity * meanFitness / (self.fitness/self.numerosity)
    
    
    #Updates------------------------------------------------------------------------------
    def updateAccuracy(self):
        """ Update the accuracy tracker """
        self.accuracy = self.correctTrack / float(self.experience)
        
        
    def updateFitness(self):
        """ Update the fitness tracker. """
        if cons.fitcalc == "accuracy":
            self.fitness = pow(self.accuracy, cons.FitnessPower) #accuracy power makes a high accuracy critically important
        else:
            print "Fitness method not recognized."


    def updateGenerality(self):
        """  """
        indCondGen = 0.0
        for j in range(len(self.condition )):
            if self.condition[j] == cons.wild:
                indCondGen += 1
        self.cl_generality = indCondGen/float(len(self.condition))
        
            
    def updateCorrectSetSize(self, numerositySum):
        """  Updates the action set size. """
        if self.experience < 1.0 / cons.beta:
            self.actionSetSize = (self.actionSetSize * (self.experience-1)+ numerositySum) / float(self.experience)
        else:
            self.actionSetSize = self.actionSetSize + cons.beta * (numerositySum - self.actionSetSize)

        return self.actionSetSize * self.numerosity

    
    def increaseExperience(self):
        """ Increases the Experience of the classifier by one. """
        self.experience = self.experience + 1


    def increaseCorrectTrack(self):
        """ Increases the correct classification tracking by one. """
        self.correctTrack = self.correctTrack + 1
        
        
    def addNumerosity(self, num):
        """ Adds to the numerosity of the classifier.
        @param num The added numerosity (can be negative!). """
        self.numerosity = self.numerosity + num
        
        
    def setAccuracy(self,acc):
        """ Sets the accuracy of the classifier """
        self.accuracy = acc
        
        
    def setFitness(self, fit):
        """  Sets the fitness of the classifier. """
        self.fitness = fit
        

    def setTimeStamp(self, ts):
        """ Sets the time stamp of the classifier. """
        self.timeStamp = ts
        
    #-------------------------------------------------------------------------------------
    
    def increaseMatch(self, isTrain):
        """ Tracks Evaluation Training and Testing matches for rule """
        if isTrain:
            self.evalTrainMatch += 1
        else:
            self.evalTestMatch += 1
        
    def increaseCorrect(self, isTrain):
        """ Tracks Evaluation Training and Testing correct classifications for rule """
        if isTrain:
            self.evalTrainCorrect += 1
        else:
            self.evalTestCorrect += 1
    
    def clearTrainEval(self):
        """ resets evaluation of train and test for each evaluation of population """
        self.evalTrainMatch = 0
        self.evalTrainCorrect = 0
        
        
    def clearTestEval(self):
        """ resets evaluation of train and test for each evaluation of population """
        self.evalTestMatch = 0
        self.evalTestCorrect  = 0
        
        
    #Return Values------------------------------------------------------------------------
    def getAccuracy(self):
        """ Returns the accuracy of the classifier. """
        return self.accuracy

        
    def getClass(self):
        """  Returns the action of the classifier. """
        return self.classification
 
 
    def getCondition(self):
        """ Returns the condition of the classifier. """
        return self.condition

        
    def getFitness(self):
        """ Returns the fitness of the classifier. """
        return self.fitness

    
    def getNumerosity(self):
        """ Returns the numerosity of the classifier. """
        return self.numerosity


    def setClassifierGenerality(self, indCondGen):
        """ Sets the classifier generality calculated outside this class, and only durring evaluations """
        self.cl_generality = indCondGen
        
    
    def getTimeStamp(self):
        """ Returns the time stamp of the classifier. """
        return self.timeStamp
    #-------------------------------------------------------------------------------------
    
    def printClassifier(self):
        """ Formats and returns an output string describing this classifier. """ #Dictionary Update
        classifierString = ""
        for x in self.condition:
            classifierString = classifierString + str(x) + "\t"
                
        if self.evalTrainMatch > 0:
            tempTrainAcc = self.evalTrainCorrect/float(self.evalTrainMatch)
        else:
            tempTrainAcc = 0
        if self.evalTestMatch > 0:
            tempTestAcc = self.evalTestCorrect/float(self.evalTestMatch)
        else:
            tempTestAcc = 0
        classifierString = classifierString + str(self.classification)+"\t"+str(self.fitness)+"\t"+str(self.accuracy)+"\t"+str(self.correctTrack)+"\t"+str(self.experience)+"\t"+str(self.numerosity)+"\t"+str(self.actionSetSize)+"\t"+str(self.timeStamp)+"\t"+str(self.cl_generality)+"\t"+str(self.evalTrainMatch)+"\t"+str(self.evalTrainCorrect)+"\t"+str(tempTrainAcc)+"\t"+str(self.evalTestMatch)+"\t"+str(self.evalTestCorrect)+"\t"+str(tempTestAcc)+"\t"+str(self.initTimeStamp)+"\n"
        return classifierString
    
