"""
Name: EK_AF_UCS_Constants.py
Authors: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA
Contact: ryan.j.urbanowicz@darmouth.edu
Created: Feb. 12, 2010
---------------------------------------------------------------------------------------------------------------------------------------------------------
EK_AF_UCS Version 1.0: An learning classifier system algorithm implemented with our new attribute tracking, feedback, and expert knowledge strategies for
modeling epistasis and heterogeneity data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

#!/usr/bin/env python
from EK_AF_UCS_Expert_Knowledge import *

class Constants:
    """ Sets and makes available all global run constants. """
    def setRunOps(self, popSize, crossVal, trackFreq, trainFile, testFile, environment, learnIterations, remakeFile, expertK, defaultRuleDelete):
        """  """
        self.popSize = popSize #[int] max size of population
        if testFile == None or testFile == 'None':
            self.crossVal = False #[Boolean] is cross validation used?
        else:
            self.crossVal = True
        self.trackFreq = trackFreq #[int] how often to print tracking summary
        self.trainFile = trainFile
        self.testFile = testFile
        self.environment = environment
        self.learnIterations = learnIterations
        self.remakeFile = remakeFile
        self.expertK = expertK
        self.defaultRuleDelete = defaultRuleDelete

    
    def setConstants(self, beta, theta_GA, pX, fitRed, theta_Sel, wild, accuracyIni, fitnessIni, p_nocare, pMut, theta_Sub, accZ, delta, theta_Del, fitnessPower):
        """  """
        self.beta = beta # [float] learning parameter
        self.theta_GA = theta_GA # GA activation experience threshold
        self.pX = pX # The probability of applying crossover in an offspring classifier.
        self.fitnessReduction = fitRed #The reduction in fitness received when GA is employed 
        self.thetaSelect = theta_Sel # The fraction of the action set to be included in tournament selection.
        self.wild = wild
        self.accuracyIni = accuracyIni
        self.fitnessIni = fitnessIni
        self.prob_dontcare = p_nocare
        self.pMut = pMut
        self.theta_sub = theta_Sub
        self.accZ = accZ
        self.delta = delta   
        self.theta_del = theta_Del
        self.FitnessPower = fitnessPower
        

    def setMech(self, subsumption, selection, covering, mutation, crossover, EK_type, p_rule_search, fitcalc, coverGenerality, attDisc, attributeTracking, attributeFeedback, EK_apply, attDiv, attributePunish,attributeID,digitsOfPrecision):
        """  """
        self.sub = subsumption #[Boolean] Do subsumption?  !!!!!!might wantto change to allow for NOne, both or either action or GA only subsumption!!!!!
        self.sel = selection #[string] type of selection
        self.cover = covering #[string] type of population initialization
        self.mut = mutation #[Boolean] use mutation operator
        self.cross = crossover #[string] use crossover? what type of crossover
        self.EK_type = EK_type #[string]
        self.EK_prob_adjust = 0.01
        self.attDisc = attDisc
        self.coverGenerality = float(coverGenerality)
        self.digitsOfPrecision = float(digitsOfPrecision)
        
        self.attributeTracking = attributeTracking
        self.attributeFeedback = attributeFeedback
        self.EK_apply = EK_apply
        self.attDiv = attDiv
        self.attributePunish = attributePunish
        self.attributeID = attributeID
        
        if p_rule_search:
            self.p_rule_search = 0.0001
        else:
            self.p_rule_search = p_rule_search
        self.fitcalc = fitcalc        

    
    def setEnvCons(self, attributeCombos, numClasses, lenTrain):
        """  """
        self.attributeCombos = attributeCombos
        self.numClasses = numClasses
        self.trainingDataSize = lenTrain
        
        
    def refEnv(self, e):
        """  """
        self.environmentObject = e
        
        
    def makeExpert(self, maximum):
        #Make an expert Knowledge object and make an attribute weight list available via cons.EKweights
        self.maxEK = maximum
        self.EK = Expert_Knowledge(self.expertK, self.environmentObject, self.coverGenerality, maximum, self.digitsOfPrecision)# coverGenerality is DCP
        self.EKListMax = max(self.EK.weightList)

        
    def getParamSummary(self):
        """  """
        if self.expertK == 'False':
            parameter_Summary = "Analysis Parameters:\n" + \
                "Train Dataset = " + str(self.trainFile) + "\n" + \
                "Test Dataset = " + str(self.testFile) + "\n" + \
                "Remake Dataset = " + str(self.remakeFile) + "\n" + \
                "Environment = " + str(self.environment) + "\n" + \
                "Cross Validation = " + str(self.crossVal) + "\n" + \
                "Tracking Frequency = " + str(self.trackFreq) + "\n" + \
                "Learning Iterations = " + str(self.learnIterations) + "\n" + \
                "Maximum Population Size = " + str(self.popSize) + "\n" + \
                "\nMechanisms:\n" + \
                "Predictive Rule Search = " + str(self.p_rule_search) + "\n" + \
                "Fitness Calculation = " + str(self.fitcalc) + "\n" + \
                "Subsumption = " + str(self.sub) + "\n" + \
                "Selection = " + str(self.sel) + "\n" + \
                "Covering = " + str(self.cover) + "\n" + \
                "Mutation = " + str(self.mut) + "\n" + \
                "Crossover = " + str(self.cross) + "\n\n" + \
                "Constants:\n" + \
                "Beta = " + str(self.beta)+ "\n" + \
                "Theta_GA = " + str(self.theta_GA) + "\n"+ \
                "Prob. Crossover = " + str(self.pX) + "\n"+ \
                "Fitness Reduction = " +str(self.fitnessReduction) + "\n"+ \
                "Theta_Select = " + str(self.thetaSelect) + "\n"+ \
                "Wild = " + str(self.wild) + "\n"+ \
                "Init. Accuracy = " +str(self.accuracyIni)+ "\n"+ \
                "Init. Fitness = " +str(self.fitnessIni)+ "\n"+ \
                "Wild Probability = " +str(self.prob_dontcare)+ "\n"+ \
                "Mutation Probability = " +str(self.pMut)+ "\n"+ \
                "Theta_Subsumption = " +str(self.theta_sub)+ "\n"+ \
                "Accuracy Z = " +str(self.accZ)+ "\n"+ \
                "Delta = " +str(self.delta)+ "\n"+ \
                "Theta_Delete = " +str(self.theta_del)+ "\n"+ \
                "Fitness (Accuracy Order) = " +str(self.FitnessPower)+ "\n"
        else:
            parameter_Summary = "Analysis Parameters:\n" + \
                "Train Dataset = " + str(self.trainFile) + "\n" + \
                "Test Dataset = " + str(self.testFile) + "\n" + \
                "Remake Dataset = " + str(self.remakeFile) + "\n" + \
                "Expert Knowledge / source = " + str(self.expertK) + "\n" + \
                "Expert Knowledge / MAX Param. = " + str(self.maxEK) + "\n" + \
                "Expert Knowledge Weights:\t" + str(self.EK.weightList) + "\n" + \
                "Expert Knowledge / DCP = " + str(self.coverGenerality) + "\n" + \
                "Environment = " + str(self.environment) + "\n" + \
                "Cross Validation = " + str(self.crossVal) + "\n" + \
                "Tracking Frequency = " + str(self.trackFreq) + "\n" + \
                "Learning Iterations = " + str(self.learnIterations) + "\n" + \
                "Maximum Population Size = " + str(self.popSize) + "\n" + \
                "\nMechanisms:\n" + \
                "Expert Knowledge Type = " + str(self.EK_type) + "\n" + \
                "Expert Knowledge Adjustment = " + str(self.EK_prob_adjust) + "\n" + \
                "Predictive Rule Search = " + str(self.p_rule_search) + "\n" + \
                "Fitness Calculation = " + str(self.fitcalc) + "\n" + \
                "Subsumption = " + str(self.sub) + "\n" + \
                "Selection = " + str(self.sel) + "\n" + \
                "Covering = " + str(self.cover) + "\n" + \
                "Mutation = " + str(self.mut) + "\n" + \
                "Crossover = " + str(self.cross) + "\n\n" + \
                "Constants:\n" + \
                "Beta = " + str(self.beta)+ "\n" + \
                "Theta_GA = " + str(self.theta_GA) + "\n"+ \
                "Prob. Crossover = " + str(self.pX) + "\n"+ \
                "Fitness Reduction = " +str(self.fitnessReduction) + "\n"+ \
                "Theta_Select = " + str(self.thetaSelect) + "\n"+ \
                "Wild = " + str(self.wild) + "\n"+ \
                "Init. Accuracy = " +str(self.accuracyIni)+ "\n"+ \
                "Init. Fitness = " +str(self.fitnessIni)+ "\n"+ \
                "Wild Probability = " +str(self.prob_dontcare)+ "\n"+ \
                "Mutation Probability = " +str(self.pMut)+ "\n"+ \
                "Theta_Subsumption = " +str(self.theta_sub)+ "\n"+ \
                "Accuracy Z = " +str(self.accZ)+ "\n"+ \
                "Delta = " +str(self.delta)+ "\n"+ \
                "Theta_Delete = " +str(self.theta_del)+ "\n"+ \
                "Fitness (Accuracy Order) = " +str(self.FitnessPower)+ "\n"
        return parameter_Summary

EK_AF_UCS_Constants = Constants()
cons = EK_AF_UCS_Constants #To access one of the above constant values from another module, import EK_AF_UCS_Constants * and use "cons.Xconstant"
