"""
Name: EK_AF_UCS_Expert_Knowledge.py
Authors: Delaney Granizo-Mackenzie and Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA
Contact: ryan.j.urbanowicz@darmouth.edu
Created: Feb. 12, 2010
---------------------------------------------------------------------------------------------------------------------------------------------------------
EK_AF_UCS Version 1.0: An learning classifier system algorithm implemented with our new attribute tracking, feedback, and expert knowledge strategies for
modeling epistasis and heterogeneity data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

#!/usr/bin/env python
#Import modules
from EK_AF_UCS_Constants import *
import math
import copy

class Expert_Knowledge:
    #Set Constructors-------------------------------------------------------------------------------------------------------   
    def __init__(self, expertK, e, DCP, max, digitsOfPrecision): # max = b
        """ EK initialization """
        self.weightList = []
        #self.mAFList = [0 for i in range(e.numAttributes)]
        
        try:
            fileObject = open(expertK, 'r')  # opens each datafile to read.
        except:
            #break
            print "EK file Not Found!"
            print expertK

        tempLine = None
        for i in range(3): #gets through garbage
            tempLine = fileObject.readline()
            
        self.headerDict = {}
        valueList = []
        #print e.numAttributes
        for i in range(e.numAttributes):
            tempLine = fileObject.readline()
            tempList = tempLine.strip().split('\t')
            self.headerDict[tempList[0]] = float(tempList[1]) #all attributes and corresponding EK scores are hashed in a dictionary
            valueList.append(float(tempList[1]))
        print self.headerDict
        #Order the list by SNP ID
        self.orderedValueList = []
        for i in range(e.numAttributes):
            self.orderedValueList.append(float(self.headerDict[e.headerList[i]]))
            
        print "Ordered EK List: " + str(self.orderedValueList) #debug
        from EK_AF_UCS_Constants import *
        if cons.EK_type == 'newLogit':
            self.weightList = self.logisticAttWeights(self.orderedValueList, DCP, max, digitsOfPrecision)
        else:
            print "Specified Expert Knowledge Transformation not found."
            
        print "Ordered Weight List: " + str(self.weightList) 


    #Delaney EK implementation###############################################################################
    def logisticAttWeights(self, rawDataList, DCP, d, digitsOfPrecision):
        """ Uses the logistic function to normalize the weights. """
        c = (1 - DCP) * len(rawDataList)
        print ("c = "+str(c))
        d = float(d)
        print ("d = "+str(d))
        print(rawDataList)

        #Step 1: Compute the range
        minE = rawDataList[0]
        maxE = rawDataList[0]
        for e in rawDataList:
            if e > maxE:
                maxE = e
            elif e < minE:
                minE = e
        print ("minE = "+str(minE))
        print ("maxE = "+str(maxE))
        rng = maxE - minE
        print ("range = "+str(rng))

        #Step 2: Shift data to non-negative
        if minE < 0:
            for i in range(len(rawDataList)):
                rawDataList[i] = rawDataList[i] - minE
            maxE = maxE - minE
            minE = 0

        #Step 3: Calculate beta
        beta = 2*math.log((1-(0.5-d))/(0.5-d))/rng
        #self.final_b = beta
        print ("beta = " +str(beta))

        #Step 4: Calculate alpha via Newton-Raphson
        alpha_calc_failed = False
        #initial guess
        alpha = -beta*(minE + rng/2)
        print("alpha starting guess = "+str(alpha))
        old_alpha = alpha-1.0
        while math.fabs(alpha-old_alpha) > pow(10,-digitsOfPrecision): #while our guess isn't close enough
            old_alpha = alpha
            try:
                alpha = alpha - ((self.logitsum(rawDataList,alpha,beta)-c)/(self.logitprimesum(rawDataList,alpha,beta)))
            except:
                alpha_calc_fail = True
                print("Failed to compute alpha.")
                break

        print ("alpha scaled for c = " +str(alpha))
        normalizedData = self.logitFinal(rawDataList, alpha, beta)
        
        sum = 0
        for e in normalizedData:
            sum = sum + e
        print("Normalized data: " + str(normalizedData))
        print("Approzimation of c = "+str(sum)+", actual requested c = "+str(c))

        return normalizedData


    def logitsum(self, rawDataList, a, b):
        """ compute the sum of the probabilities. """
        sum = 0.0
        for i in range(len(rawDataList)):
            sum += self.logit(rawDataList[i],a,b)
        return sum
    
    
    def logit(self, rawData, a, b):
        """ Do logistic regression on a datum """
        return 1/(1 + math.exp(-(a+b*rawData)))
    
    
    def logitprimesum(self, rawDataList, a, b):
        """  Compute the derivative of the sum of the probabilities. """
        sum = 0.0
        for i in range(len(rawDataList)):
            sum += self.logitprime(rawDataList[i],a,b)
        return sum
       
        
    def logitprime(self, rawData, a, b):
        """ Take the derivative of logit on a datum. """
        return self.logit(rawData,a,b)*(1-self.logit(rawData,a,b))
    

    def logitFinal(self, rawDataList, a, b):
        """ Do logistic regression on the data. """
        selectionP = []
        for i in range(len(rawDataList)):
            selectionP.append(self.logit(rawDataList[i],a,b))
        return selectionP
    ########################################################################################end Delany
    
