"""
Name: EK_AF_UCS_PredictionArray.py
Authors: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA
Contact: ryan.j.urbanowicz@darmouth.edu
Created: Feb. 12, 2010
---------------------------------------------------------------------------------------------------------------------------------------------------------
EK_AF_UCS Version 1.0: An learning classifier system algorithm implemented with our new attribute tracking, feedback, and expert knowledge strategies for
modeling epistasis and heterogeneity data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

#!/usr/bin/env python
#Import modules
import sys
import random

class PredictionArray:
    def __init__(self, set, size):
        """ Constructs the voting array """
        self.vote = [0.0 for i in range(size)]
        for cl in set.clSet:
            self.vote[int(cl.getClass())] += cl.getFitness()
            
            
    def getBestValue(self):
        """ Returns the highest value in the prediction array. """
        return max(self.vote)
    
        
    def getValue(self, i):
        """ Returns the value of the specified entry in the prediction array. """
        if i>=0 and i<len(self.vote):
            return self.vote[i]
        return -1.0
    
    
    #Action Selection Methods -----------------------------------------------------------
    def bestClassWinner(self):
        """ Selects the action in the prediction array with the best value.
         *MODIFIED so that in the case of a tie between actions - an action is selected randomly between the tied highest actions. """
        highVal = 0.0
        for i in self.vote:
            if i > highVal:
                highVal = i
        bestIndexList = []
        for i in range(len(self.vote)):
            if self.vote[i] == highVal:
                bestIndexList.append(i)
        if highVal == 0.0:
            return None
        elif len(bestIndexList) > 1:
            return "tie"
        else:
            return bestIndexList[0]

