"""
Name: EK_AF_UCS_TimeKeeper.py
Authors: Ryan Urbanowicz - written at Dartmouth College, Hanover, NH, USA
Contact: ryan.j.urbanowicz@darmouth.edu
Created: Feb. 12, 2010
---------------------------------------------------------------------------------------------------------------------------------------------------------
EK_AF_UCS Version 1.0: An learning classifier system algorithm implemented with our new attribute tracking, feedback, and expert knowledge strategies for
modeling epistasis and heterogeneity data mining and classification tasks.
Copyright (C) 2013 Ryan Urbanowicz 
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful but WITHOUT ANY WARRENTY; without even the implied warranty of MERCHANTABLILITY 
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, 
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------------------------------------------------------------------------------------------
"""

#!/usr/bin/env python
import time

class Timer:
    def __init__(self):
        """ Initializes all Timer values for the algorithm """        
        # Global Time objects
        self.globalStartRef = time.time()
        self.globalTime = 0.0
        self.addedTime = 0.0
        
        # Selection Time Variables
        self.startRefSelection = 0.0
        self.globalSelection = 0.0
         
        # GA Operators Time Variables
        self.startRefGA = 0.0 #global GA run time
        self.globalGA = 0.0
        
        self.startRefMutation = 0.0
        self.globalMutation = 0.0
        
        self.startRefCrossover = 0.0
        self.globalCrossover = 0.0
        
        # Deletion Time Variables
        self.startRefDeletion = 0.0
        self.globalDeletion = 0.0
        
        # Match Time Variables
        self.startRefMatchSet = 0.0
        self.globalMatchSet = 0.0
        
        # Subsumption Time Variables
        self.startRefSubsumption = 0.0
        self.globalSubsumption = 0.0
        
        # Evaluation Time Variables
        self.startRefEvaluation = 0.0
        self.globalEvaluation = 0.0  
        
        #Debug Time Variables
        self.startRefDebugA = 0.0
        self.globalDebugA = 0.0
        self.startRefDebugB = 0.0
        self.globalDebugB = 0.0

        # Expert Knowledge Time Objects
        self.startRefExpertInit = 0.0
        self.globalExpertInit = 0.0            
    
        # Rule Compaction
        self.startRefRuleCmp = 0.0
        self.globalRuleCmp = 0.0
    
    def setTimerRestart(self, remakeFile):
        """ Sets all time values to the those previously evolved in the loaded popFile.  """
        try:
            fileObject = open(remakeFile+"_PopStats.txt", 'r')  # opens each datafile to read.
        except:
            #break
            print "Data-set Not Found!"

        tempLine = None
        #for i in range(14): #original before added AccSum to output file.
        for i in range(18):
            tempLine = fileObject.readline()
        tempList = tempLine.strip().split('\t')
        self.addedTime = float(tempList[1]) * 60
        
        tempLine = fileObject.readline()
        tempList = tempLine.strip().split('\t') 
        self.globalGA = float(tempList[1]) * 60
        
        tempLine = fileObject.readline()
        tempList = tempLine.strip().split('\t') 
        self.globalSelection = float(tempList[1]) * 60    
 
        tempLine = fileObject.readline()
        tempList = tempLine.strip().split('\t') 
        self.globalCrossover = float(tempList[1]) * 60

        tempLine = fileObject.readline()
        tempList = tempLine.strip().split('\t') 
        self.globalMutation = float(tempList[1]) * 60

        tempLine = fileObject.readline()
        tempList = tempLine.strip().split('\t') 
        self.globalDeletion = float(tempList[1]) * 60
        
        tempLine = fileObject.readline()
        tempList = tempLine.strip().split('\t') 
        self.globalMatchSet = float(tempList[1]) * 60
        
        tempLine = fileObject.readline()
        tempList = tempLine.strip().split('\t') 
        self.globalSubsumption = float(tempList[1]) * 60
        
        tempLine = fileObject.readline()
        tempList = tempLine.strip().split('\t') 
        self.globalEvaluation = float(tempList[1]) * 60
                 
        tempLine = fileObject.readline()
        tempList = tempLine.strip().split('\t') 
        self.globalRuleCmp = float(tempList[1]) * 60
                
        tempLine = fileObject.readline()
        tempList = tempLine.strip().split('\t') 
        self.globalExpertInit = float(tempList[1]) * 60
            
        tempLine = fileObject.readline()
        tempLine = fileObject.readline()
        tempList = tempLine.strip().split('\t') 
        self.globalDebugA = float(tempList[1]) * 60
        
        tempLine = fileObject.readline()
        tempList = tempLine.strip().split('\t') 
        self.globalDebugB = float(tempList[1]) * 60
        
        fileObject.close()
        print "RESTART RUN TIME SUMMARY(in minutes):"
        print "Global Time = " + str(self.endGlobalTimer())#Reports time in minutes
        print "Global GA Time = " + str(self.globalGA/ 60.0)#Reports time in minutes
        print "Selection Time = " + str(self.globalSelection/ 60.0)#Reports time in minutes
        print "Crossover Time = " + str(self.globalCrossover/ 60.0)#Reports time in minutes
        print "Mutation Time = " + str(self.globalMutation/ 60.0)#Reports time in minutes
        print "Deletion Time = " + str(self.globalDeletion/ 60.0)#Reports time in minutes
        print "MatchSet Time = " + str(self.globalMatchSet/ 60.0)#Reports time in minutes
        print "Subsumption Time = " + str(self.globalSubsumption/ 60.0)#Reports time in minutes
        print "Evaluation Time = " + str(self.globalEvaluation/ 60.0)#Reports time in minutes
        print "Rule Compaction Time\t"+str(self.globalRuleCmp/ 60.0)
        print "Expert Init. Time = " + str(self.globalExpertInit/ 60.0)#Reports time in minutes
        print "DebugA Time = " + str(self.globalDebugA/ 60.0)#Reports time in minutes
        print "DebugB Time = " + str(self.globalDebugB/ 60.0)#Reports time in minutes

        
    # ************************************************************
    def startTimeGA(self):
        """ Tracts GA Time """
        self.startRefGA = time.time()
        
    def stopTimeGA(self):
        """ Tracks GA Time """
        diff = time.time() - self.startRefGA
        self.globalGA += diff
        
    def startTimeSelection(self):
        """ Tracks Selection Time """
        self.startRefSelection = time.time()
        
    def stopTimeSelection(self):
        """ Tracks Selection Time """
        diff = time.time() - self.startRefSelection
        self.globalSelection += diff
        
    def startTimeMutation(self):
        """ Tracks Mutation Time """
        self.startRefMutation = time.time()
        
    def stopTimeMutation(self):
        """ Tracks Mutation Time """
        diff = time.time() - self.startRefMutation
        self.globalMutation += diff
         
    def startTimeCrossover(self):
        """ Tracks Crossover Time """
        self.startRefCrossover = time.time() 
               
    def stopTimeCrossover(self):
        """ Tracks Crossover Time """
        diff = time.time() - self.startRefCrossover
        self.globalCrossover += diff
         
    def startTimeDeletion(self):
        """ Tracks Deletion Time """
        self.startRefDeletion = time.time()
        
    def stopTimeDeletion(self):
        """ Tracks Deletion Time """
        diff = time.time() - self.startRefDeletion
        self.globalDeletion += diff
        
    def startTimeMatchSet(self):
        """ Tracks MatchSet Time """
        self.startRefMatchSet = time.time()
         
    def stopTimeMatchSet(self):
        """ Tracks MatchSet Time """
        diff = time.time() - self.startRefMatchSet
        self.globalMatchSet += diff       
    
    def startTimeSubsumption(self):
        """Tracks Subsumption Time """
        self.startRefSubsumption = time.time()

    def stopTimeSubsumption(self):
        """Tracks Subsumption Time """
        diff = time.time() - self.startRefSubsumption
        self.globalSubsumption += diff       
        
    def startTimeEvaluation(self):
        """ Tracks Evaluation Time """
        self.startRefEvaluation = time.time()
        
    def stopTimeEvaluation(self):
        """ Tracks Evaluation Time """
        diff = time.time() - self.startRefEvaluation
        self.globalEvaluation += diff
          
    def startTimeDebugA(self):
        """  """
        self.startRefDebugA = time.time()   
         
    def stopTimeDebugA(self):
        """  """
        diff = time.time() - self.startRefDebugA
        self.globalDebugA += diff
        
    def startTimeDebugB(self):
        """  """
        self.startRefDebugB = time.time()   
         
    def stopTimeDebugB(self):
        """  """
        diff = time.time() - self.startRefDebugB
        self.globalDebugB += diff
           
    def startTimeExpertInit(self):
        """  """
        self.startRefExpertInit = time.time()   
         
    def stopTimeExpertInit(self):
        """  """
        diff = time.time() - self.startRefExpertInit
        self.globalExpertInit += diff
        
    def startTimeRuleCmp(self):
        """  """
        self.startRefRuleCmp = time.time()   
         
    def stopTimeRuleCmp(self):
        """  """
        diff = time.time() - self.startRefRuleCmp
        self.globalRuleCmp += diff
        
    # ******************************************************       
    def endGlobalTimer(self):
        """ Set the global end timer, call at very end of algorithm. """
        self.globalTime = (time.time() - self.globalStartRef) + self.addedTime #Reports time in minutes
        return self.globalTime/ 60.0
        
        
    def reportTimes(self):
        """ Reports the time summaries for this run. Returns a string ready to be printed out."""
        outputTime = "Global Time\t"+str(self.globalTime/ 60.0)+ \
        "\nGlobal GA Time\t"+str(self.globalGA/ 60.0)+ \
        "\nSelection Time\t"+str(self.globalSelection/ 60.0)+ \
        "\nCrossover Time\t" + str(self.globalCrossover/ 60.0)+ \
        "\nMutation Time\t" + str(self.globalMutation/ 60.0)+ \
        "\nDeletion Time\t" + str(self.globalDeletion/ 60.0)+ \
        "\nMatchSet Time\t" + str(self.globalMatchSet/ 60.0)+ \
        "\nSubsumption Time\t" + str(self.globalSubsumption/ 60.0)+ \
        "\nEvaluation Time\t"+str(self.globalEvaluation/ 60.0) + \
        "\nRule Compaction Time\t"+str(self.globalRuleCmp/ 60.0) + \
        "\nExpert Init. Time\t"+str(self.globalExpertInit/ 60.0) + \
        "\n\nDebugA Time\t"+str(self.globalDebugA/ 60.0) + \
        "\nDebugB Time\t"+str(self.globalDebugB/ 60.0) + "\n" 
 
        
        print "GLOBAL RUN TIME SUMMARY(in minutes):"
        print "Global Time = " + str(self.globalTime/ 60.0)#Reports time in minutes
        #print "Global GA Time = " + str(self.globalGA/ 60.0)#Reports time in minutes
        #print "Selection Time = " + str(self.globalSelection/ 60.0)#Reports time in minutes
        #print "Crossover Time = " + str(self.globalCrossover/ 60.0)#Reports time in minutes
        #print "Mutation Time = " + str(self.globalMutation/ 60.0)#Reports time in minutes
        #print "Deletion Time = " + str(self.globalDeletion/ 60.0)#Reports time in minutes
        #print "MatchSet Time = " + str(self.globalMatchSet/ 60.0)#Reports time in minutes
        #print "Subsumption Time = " + str(self.globalSubsumption/ 60.0)#Reports time in minutes
        #print "Evaluation Time = " + str(self.globalEvaluation/ 60.0)#Reports time in minutes
        #print "DebugA Time = " + str(self.globalDebugA/ 60.0)#Reports time in minutes
        #print "DebugB Time = " + str(self.globalDebugB/ 60.0)#Reports time in minutes
        #print "Expert Init. Time = " + str(self.globalExpertInit/ 60.0)#Reports time in minutes
        
        return outputTime
